import re

import mwparserfromhell as mwp

DOI_RE = re.compile(r'\b(10\.\d+/[^\s\|\]\}]+)', re.I)

TAGS_RE = re.compile(r'</?\s*(ref|span|div|table|h[1-6]|b|ins|del)\b[^>\n\r]*>', re.I)

def extract(text):
    for match in DOI_RE.finditer(text):
        id = re.sub(TAGS_RE, "", match.group(1))
        yield ("doi", id)


def extract_mwp(text):
    no_tags = mwp.parse(text).strip_code()
    for match in DOI_RE.finditer(no_tags):
        yield ("doi", id)



"""
Extracts academic citations from articles from the history of Wikipedia articles
by processing a pages-meta-history XML dump and matching regular expressions
to revision content.

Currently supported identifies include:
 * PubMed
 * DOI

Usage:
    extract_cites -h | --help
    extract_cites <dump_path>...

Options:
    -h --help        Shows this documentation
"""
from itertools import chain

import docopt
from mw import xml_dump

from .extractors import doi, pubmed


def main():
    args = docopt.docopt(__doc__)
    
    def process_dump(dump, path):
        
        for page in dump:
            if page.namespace != 0: continue
            
            id_appearance = {}
            ids = set()
            for revision in page:
                
                ids = set(doi.extract(revision.text)) + \
                      set(pubmed.extract(revision.text))
                
                for id in ids:
                    if id not in id_appearance:
                       id_appearance[id] = (revision.id, revision.timestamp)
                
            for id in ids: #For the ids in the last version of the page
                appearance_id, appearance_timestamp = id_appearance[id]
                yield (page.id, page.title, appearance_id, appearance_timestamp, id[0], id[1])
                
            
        
    
    print("page_id\tpage_title\trev_id\ttimestamp\ttype\tid")
    
    id_metas = xml_dump.map(args['<dump_path>'], process_dump)
    for page_id, title, rev_id, timestamp, type, id in id_metas:
        
        print("{0}\t{1}\t{2}\t{3}\t{4}\t{5}"\
              .format(page_id,
                      title.replace("\t", "\\t").replace("\n", "\\n"),
                      rev_id,
                      timestamp.long_format(),
                      type, id))



from nose.tools import eq_

from .. import doi


def test_extract():
    
    text = """
    This is a doi randomly placed in the text 10.0000/m1
    Here's a typo that might be construed as a doi 10.60 people were there.
    {{cite|...|doi=10.0000/m2|pmid=10559875}}
    <ref>Halfaker, A., Geiger, R. S., Morgan, J. T., & Riedl, J. (2012).
    The rise and decline of an open collaboration system: How Wikipedia’s
    reaction to popularity is causing its decline.
    American Behavioral Scientist,
    0002764212469365 doi: 10.1177/0002764212469365</ref>
    """
    ids = list(doi.extract(text))
    expected = [
        ('doi', "10.0000/m1"),
        ('doi', "10.0000/m2"),
        ('doi', "10.1177/0002764212469365")
    ]
    print(ids)
    print(expected)
    eq_(ids, expected)
